%% 
% Run_Demo--Demo code to evaluate the proposed Robust Estimation of Approximate 
% Infinite Dimensional Gaussian (RAID-G) using the Flickr material database.
% For  theoretical and technical details, please refer to the following paper:
%
% Qilong Wang, Peihua Li, Wangmeng Zuo, and Lei Zhang. RAID-G: Robust Estimation of 
% Approximate Infinite Dimensional Gaussian with Application to Material Recognition. 
% IEEE Conference on Computer Vision and Pattern Recognition Pattern Recognition (CVPR), 2016.
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Qilong Wang  (Email:  qlwang at mail dot dlut dot edu dot cn), 
%                                               Peihua  Li (Email: peihuali at dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

clc;
root = pwd;
addpath(fullfile(root, 'feaextra'));
addpath(fullfile(root, 'common'));
addpath(fullfile(root, 'classifier'));
addpath('matconvnet\model');
addpath('matconvnet\matlab');
addpath('vlfeat-0.9.18\toolbox');
vl_setupnn;
vl_setup;
sd = genseed; 

expms = {'hell','homo'};

for l = 1:length(expms)

expm = expms{l};    
s = RandStream('mt19937ar','Seed',sd); 
RandStream.setGlobalStream(s);

total_start_time = round(clock);

exprs = plow_expriments('10',expm);
feat_prms = plow_features(expm);

%% Extract  features of every image per category
fprintf('****************************************************************\n');
fprintf('Extract features of every image per category\n');
fprintf('****************************************************************\n');
start_time = round(clock);

categories_dir0 = dir(exprs.base_image_dir);
categories_dir = struct([]);
for i=1:length(categories_dir0)
    if strcmp(categories_dir0(i).name, '.') == 1 ||  strcmp(categories_dir0(i).name, '..') == 1 || categories_dir0(i).isdir == 0
        continue;
    end
    categories_dir = [categories_dir; categories_dir0(i)];
end

featperclass = cell(length(categories_dir),1); 

for i=1: length(categories_dir)
   
    image_dir = fullfile(exprs.base_image_dir, sprintf('%s', categories_dir(i).name));
    data_dir   =  fullfile(exprs.base_feature_dir,  exprs.image_dataset,   sprintf('%s', categories_dir(i).name));
    raw_data_dir   =  fullfile(exprs.raw_feature_dir,  exprs.image_dataset,   sprintf('%s', categories_dir(i).name));
    featperclass{i} = FeatureExtraction(image_dir,data_dir,raw_data_dir,feat_prms); 
    
end
end_time = round(clock);
elapsed_time = end_time - start_time;
fprintf('Time taken: %5d days %5d hours %5d minutes %5d seconds\n\n', elapsed_time(3), elapsed_time(4), elapsed_time(5), elapsed_time(6));


%% Split training set and test set, and compute  kernel matrix and peform SVM training
fprintf('****************************************************************\n');
fprintf('Peform SVM training and test\n');
fprintf('****************************************************************\n');

iter_max = 5;
start_time = round(clock);
accuracies = zeros(iter_max, 1);

for iter = 1: iter_max
    
     [trainlist,testlist] = random_split_training_test(exprs,categories_dir);
     [traindata, testdata, trainlabel, testlabel] =  get_data_label(categories_dir,featperclass,trainlist,testlist);
     
     accuracy = SVMforClassification(categories_dir,traindata,testdata,trainlabel,testlabel,exprs);
     accuracies(iter) = accuracy;
     
end

fprintf('Average accuracy of RIAD-G with %s kernel on FMD over %d times is %f+%f \n', expm, iter_max, mean(accuracies), std(accuracies));
end_time = round(clock);
elapsed_time = end_time - start_time;
fprintf('Time taken: %5d days %5d hours %5d minutes %5d seconds\n\n', elapsed_time(3), elapsed_time(4), elapsed_time(5), elapsed_time(6));
total_end_time = round(clock);
fprintf('Total time of the classification program is:\n');

disp(total_start_time);
disp(total_end_time);
end