function  features = FeatureExtraction(image_dir,data_dir,raw_data_dir,feat_prms) 

% FeatureExtraction computes the RIAD-G for all images
%
% Qilong Wang, Peihua Li, Wangmeng Zuo, and Lei Zhang. RAID-G: Robust Estimation of 
% Approximate Infinite Dimensional Gaussian with Application to Material Recognition. 
% IEEE Conference on Computer Vision and Pattern Recognition Pattern Recognition (CVPR), 2016.
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Qilong Wang  (Email:  qlwang at mail dot dlut dot edu dot cn), 
%                                               Peihua  Li (Email: peihuali at dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

fnames = dir(fullfile(image_dir, '*.jpg'));
imageFileList = cell( length(fnames),1);
for f = 1: length(fnames)
    imageFileList{f} = fnames(f).name;
end

if strcmp(feat_prms.fmap,'homo')
    Rfeat_dim = 512*3;
elseif strcmp(feat_prms.fmap,'hell')
    Rfeat_dim = 512;
end

if strcmp(feat_prms.feature_type,'gaussian')==1
    ddim = Rfeat_dim+1;
    single_desc_dim = ddim*(ddim+1)/2;
elseif strcmp(feat_prms.feature_type,'mean')==1 
    single_desc_dim = Rfeat_dim;
else
    single_desc_dim = (Rfeat_dim*(Rfeat_dim+1)/2);
end

if strcmp(feat_prms.fmap,'homo')
    feat_dim = single_desc_dim;
elseif strcmp(feat_prms.fmap,'hell')
    feat_dim = single_desc_dim*length(feat_prms.scale);
end

features = zeros(feat_dim, numel(imageFileList), 'single');
net = load(feat_prms.cnnmodel);

if strcmp(feat_prms.cnnmodel,'imagenet-vgg-f.mat') == 1    
    net.layers = net.layers(1:14);
elseif strcmp(feat_prms.cnnmodel,'imagenet-vgg-verydeep-16.mat') == 1       
    net.layers = net.layers(1:31);
elseif strcmp(feat_prms.cnnmodel,'imagenet-vgg-verydeep-19.mat') == 1
    net.layers = net.layers(1:37);
else
    net.layers = net.layers(1:15);
end
    
if feat_prms.GPUmode
    net = vl_simplenn_move(net, 'gpu') ;
end

shift = feat_prms.constrant.*eye(Rfeat_dim);
true_mat = true(Rfeat_dim,Rfeat_dim);
in_triu = triu(true_mat);

fprintf('Extracting %s features--Processing %s', feat_prms.feature_type,image_dir);

for f =1:size(imageFileList,1)

    if ~mod(f, 5)
        fprintf('.');
    end
    
    curr_filename = imageFileList{f};

    suffix_pos = findstr(curr_filename, '.jpg');
    outFName = fullfile(data_dir, [curr_filename(1:suffix_pos-1) '.mat']);
    
    if feat_prms.canSkip && size(dir(outFName), 1)
        load(outFName);
        features(:,f) = Fea_each_im;
        continue;
    end
    
    imageFName = fullfile(image_dir, curr_filename);
    I = imread(imageFName);
    
    if size(I,3) < 3
         I = repmat(I, [1 1 3]);
    end  
    % resize image
    [im_h, im_w, ~] = size(I);
    
    if max(im_h, im_w) > feat_prms.maxImageSize,
        I = imresize(I, feat_prms.maxImageSize/max(im_h, im_w), 'bicubic');
        [im_h, im_w] = size(I);
    end;
    if min(im_h, im_w) < feat_prms.minImageSize,
        I = imresize(I, feat_prms.minImageSize/min(im_h, im_w), 'bicubic');
        [im_h, im_w] = size(I);
    end;
    
    RawFeas = GenerateRawFeaVGGCNN(I,net,raw_data_dir,curr_filename(1:suffix_pos-1),feat_prms);
    Fea_each_im = IndividualModelIm_VGG(RawFeas,feat_prms,single_desc_dim,in_triu,shift);
            
    %% Re-Normalization 
    if feat_prms.renorm
        Fea_each_im = squash_features(Fea_each_im,'l2');
    end
    Fea_each_im = single(Fea_each_im);
    features(:,f) = Fea_each_im;
    sp_make_dir(outFName);
    save(outFName, 'Fea_each_im');   
    
end
fprintf('Completed.\n');

