function  RawD = GenerateRawFeaVGGCNN(I,net,raw_data_dir,filename,feat_prms) 

% GenerateRawFeaVGGCNN generates the mutli-scale CNN features for RIAD-G 
%
% For  theoretical and technical details, please refer to the following paper:
%
% Qilong Wang, Peihua Li, Wangmeng Zuo, and Lei Zhang. RAID-G: Robust Estimation of 
% Approximate Infinite Dimensional Gaussian with Application to Material Recognition. 
% IEEE Conference on Computer Vision and Pattern Recognition Pattern Recognition (CVPR), 2016.
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Qilong Wang  (Email:  qlwang at mail dot dlut dot edu dot cn), 
%                                               Peihua  Li (Email: peihuali at dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

Fea_Raw = [];
Fea_Raw.D = [];
Fea_Raw.idx = zeros(length(feat_prms.scale)*2,1);

outFName = fullfile(raw_data_dir, [filename '.mat']);
if feat_prms.canSkip && size(dir(outFName), 1)
     load(outFName);
     RawD = Fea_Raw;
     return;
end

count = 0;
for s = 1:length(feat_prms.scale)
     
    I_s = imresize(I, [size(I,1)*feat_prms.scale(s) size(I,2)*feat_prms.scale(s)]) ;
    im_ = single(I_s) ; % note: 255 range
    im_mean = imresize(net.normalization.averageImage, [size(im_,1) size(im_,2)]) ;
    im_ = im_ - im_mean  ;
    if feat_prms.GPUmode
         im_ = gpuArray(im_) ;
    end
    res = vl_simplenn(net, im_) ;
    
    if strcmp(feat_prms.cnnmodel,'imagenet-vgg-f.mat') == 1     
       TempRes = res(15).x;  %% imagenet-vgg-s
    elseif strcmp(feat_prms.cnnmodel,'imagenet-vgg-verydeep-16.mat') == 1       
       TempRes = res(32).x;  %% imagenet-vgg-verydeep-16
    elseif strcmp(feat_prms.cnnmodel,'imagenet-vgg-verydeep-19.mat') == 1
       TempRes = res(38).x;  %% imagenet-vgg-verydeep-19
    else
       TempRes = res(16).x;  %% others
    end
    
    Fea_RawT = reshape(TempRes,size(TempRes,1)*size(TempRes,2),size(TempRes,3));
    Fea_Raw.D = [Fea_Raw.D; Fea_RawT];
    Fea_Raw.idx(2*(s-1)+1) = count + 1;
    Fea_Raw.idx(2*s) = count + size(Fea_RawT,1);
    count = count + size(Fea_RawT,1);
end

sp_make_dir(outFName);
save(outFName, 'Fea_Raw');   
     
RawD = Fea_Raw;