function  Fea_each_im = IndividualModelIm_VGG(rawD,feat_prms,single_desc_dim,in_triu,shift)

% IndividualModelIm_VGG represents each image by using RIAD-G 
%
% For theoretical and technical details, please refer to the following paper:
%
% Qilong Wang, Peihua Li, Wangmeng Zuo, and Lei Zhang. RAID-G: Robust Estimation of 
% Approximate Infinite Dimensional Gaussian with Application to Material Recognition. 
% IEEE Conference on Computer Vision and Pattern Recognition Pattern Recognition (CVPR), 2016.
% 
% Please cite the paper above if you use the code:
%
% For questions,  please conact:  Qilong Wang  (Email:  qlwang at mail dot dlut dot edu dot cn), 
%                                               Peihua  Li (Email: peihuali at dlut dot edu dot cn) 
%
% The software is provided ''as is'' and without warranty of any kind,
% experess, implied or otherwise, including without limitation, any
% warranty of merchantability or fitness for a particular purpose.

if strcmp(feat_prms.fmap,'homo')
    Fea_each_im = zeros(single_desc_dim,1);
elseif strcmp(feat_prms.fmap,'hell')
    Fea_each_im = zeros(length(feat_prms.scale)*single_desc_dim,1);
end

counter = 1;
for s = 1:length(feat_prms.scale)

    ids = rawD.idx(2*(s-1)+1);
    ide = rawD.idx(2*s);
    theD = rawD.D(ids:ide,:);
    
    theD = theD';
    
    if strcmp(feat_prms.fmap,'homo')
        theD = vl_homkermap(theD, 1, 'kchi2');
    elseif strcmp(feat_prms.fmap,'hell')
        theD = squash_features(theD,'power');
    end

    range = counter:counter+single_desc_dim-1;
    
    if strcmp(feat_prms.feature_type,'mean')==1
        regionD = mean(theD');
        regionD = regionD';
    elseif strcmp(feat_prms.feature_type,'cov_mean')==1
         % second-order pooling
        regionD = real(logm(((1/size(theD,2)).*(theD *theD')) + shift)); 
        regionD = regionD(in_triu);              
    elseif strcmp(feat_prms.feature_type,'cov')==1
        DataD = cov(theD') + shift;
        [U S V] = svd(DataD);
        diag_S = diag(S);
        diag_S = log(diag_S);  
        regionD = U*diag(diag_S)*U';
        regionD = regionD(in_triu);   
    elseif strcmp(feat_prms.feature_type,'gaussian')==1
        
        true_mat = true(size(theD,1)+1,size(theD,1)+1); 
        in_triu = triu(true_mat);
        regionD = ones(size(theD,1)+1); 
        alpha = feat_prms.alpha;
        gama = (1-alpha)/(2*alpha);
        b = feat_prms.embedding_beta;
        b2 = b^2;
                
        COVD = cov(theD') + shift;              
        [U S V] = svd(COVD);
        diag_S = diag(S);      
        diag_S = sign(diag_S).*((gama.^2+abs(diag_S/alpha)).^(0.55)-gama);
        COVD = U*(diag(diag_S))*U'; 
  
        regionD(1:size(theD,1),1:size(theD,1)) = (COVD + b2.*(mean(theD,2)*mean(theD,2)')); 
        regionD(1:size(theD,1),1+size(theD,1)) =  b.*mean(theD,2);
        regionD(1+size(theD,1),1:size(theD,1)) =  b.*mean(theD,2)';
              
        [U, S, V] = svd(regionD);         
        diag_S = diag(S);
        diag_S = diag_S + feat_prms.constrantGaussian;
        log_diag_S = sign(diag_S).*abs(diag_S).^(0.9);
        regionD = U*(diag(log_diag_S))*U'; 
        regionD = regionD(in_triu);

    end

    if strcmp(feat_prms.fmap,'homo')
        Fea_each_imT = regionD(:);
        Fea_each_imT = squash_features(Fea_each_imT,'l2');
        Fea_each_im = Fea_each_im + feat_prms.scaleW(s)*Fea_each_imT;
    elseif strcmp(feat_prms.fmap,'hell')
        Fea_each_im(range) = regionD(:);
    end

    counter = counter + single_desc_dim;
end